package com.io.yutian.aulib.sql.action;

import com.io.yutian.aulib.sql.api.action.SQLUpdateBatchAction;
import com.io.yutian.aulib.sql.manager.SQLManagerImpl;
import org.jetbrains.annotations.NotNull;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

public class SQLUpdateBatchActionImpl
        extends AbstractSQLAction<List<Integer>>
        implements SQLUpdateBatchAction {

    protected final List<String> sqlContents = new ArrayList<>();

    public SQLUpdateBatchActionImpl(@NotNull SQLManagerImpl manager, @NotNull String sql) {
        super(manager, sql);
        this.sqlContents.add(sql);
    }

    @Override
    public @NotNull List<String> getSQLContents() {
        return this.sqlContents;
    }

    @Override
    public SQLUpdateBatchAction addBatch(@NotNull String sql) {
        Objects.requireNonNull(sql, "sql could not be null");
        this.sqlContents.add(sql);
        return this;
    }

    @Override
    public @NotNull List<Integer> execute() throws SQLException {
        debugMessage(new ArrayList<>());

        try (Connection connection = getManager().getConnection()) {

            try (Statement statement = connection.createStatement()) {

                for (String content : this.sqlContents) {
                    statement.addBatch(content);
                }

                int[] executed = statement.executeBatch();

                return Arrays.stream(executed).boxed().collect(Collectors.toList());
            }

        }
    }

}
